from pikanetwork.models import *


async def build_player(player_data: dict) -> Player:

    ranks = []

    friends = []

    for rank in player_data["ranks"]:
        ranks.append(GameRank(
            name=rank["name"],
            display_name=rank["displayName"],
            server=rank["server"],
            season=rank["season"]
        ))

    for friend_data in player_data["friends"]:

        friend = Friend(
            username=friend_data["username"],
            last_seen=friend_data["lastSeen"],
            online=friend_data["online"]
        )
        friends.append(friend)

    if player_data["clan"] is None:
        clan = None
    else:
        clan_data = player_data["clan"]

        members = []

        for member_raw_data in clan_data["members"]:
            member_data = member_raw_data["user"]
            member = ClanMember(
                username=member_data["username"],
                last_seen=member_data["lastSeen"],
                online=member_data["online"],
                join_date=member_raw_data["joinDate"]
            )
            members.append(member)

        clan = Clan(
            name=clan_data["name"],
            tag=clan_data["tag"],
            current_trophies=clan_data["currentTrophies"],
            creation_time=clan_data["creationTime"],
            members=members,
            owner=ClanOwner(
                username=clan_data["owner"]["username"],
                last_seen=clan_data["owner"]["lastSeen"],
                online=clan_data["owner"]["online"]
            ),
            level=clan_data["leveling"]["level"],
            experience=clan_data["leveling"]["exp"],
            total_experience=clan_data["leveling"]["totalExp"]
        )

    player = Player(
        username=player_data["username"],
        friend_status=player_data["friendStatus"],
        discord_verified=player_data["discord_verified"],
        last_seen=player_data["lastSeen"],
        ranks=ranks,
        email_verified=player_data["email_verified"],
        discord_boosting=player_data["discord_boosting"],
        clan=clan,
        rank=Rank(
            level=player_data["rank"]["level"],
            experience=player_data["rank"]["experience"],
            percentage=player_data["rank"]["percentage"],
            rank_display=player_data["rank"]["rankDisplay"]
        ),
        friends=friends
    )

    return player


async def build_player_leaderboard(leaderboard_data: dict) -> PlayerLeaderboard:

    attributes = {}

    for key in leaderboard_data:
        data = leaderboard_data[key]

        name = key.replace(" ", "_").lower()

        if data["entries"] is None:
            attributes[name] = None
        else:
            place = data["entries"][0]["place"]
            value = data["entries"][0]["value"]
            username = data["entries"][0]["id"]

            attributes[name] = PlayerLeaderboardBase(
                total_players=data["metadata"]["total"],
                place=place,
                value=value,
                username=username
            )

    return PlayerLeaderboard(**attributes)



# -*- coding: utf-8 -*-

"""
PikaNetwork.py
~~~~~~~~~
:license: MIT, see LICENSE for more details.
"""

# Meta.

__title__ = 'pikanetwork.py'
__author__ = 'LetsChill'
__copyright__ = 'LetsChill 2023'
__license__ = 'MIT'

__version__ = '1.0.2'

# structure

from pikanetwork.api import PikaAPI



class InvalidName(ValueError):
    """Raised when the name given is invalid"""
    pass


class PlayerNotFound(Exception):
    pass


class PlayerWithNoClan(Exception):
    pass

class InvalidServerArgument(Exception):
    pass


import typing

import aiohttp
from pikanetwork.builders import build_player,build_player_leaderboard
from .exceptions import *
from .models import (Player, PlayerLeaderboard)
import logging
import json
import sys

ALLOWED_SERVERS = [
    "opfactions",
    "lifesteal",
    "oplifesteal",
    "classicskyblock",
    "opskyblock",
    "opprison",
    "survival",
    "kitpvp",
    "practice",
    "skywars",
    "bedwars"
]

async def fetch(client, url):
    async with client.get(url) as resp:
        if resp.status == 400:
            raise PlayerNotFound(f"The player {name} does not exist in the database.")

        return await resp.text()


class PikaAPI:
    """
    The base class of the pikanetwork api wrapper
    """
    def __init__(self):
        self.logger = logging.getLogger(__name__)

    async def get_player(self, name: str) -> Player:
        """
        Returns a player object

        used to get all the data about a player in-game.
        """
        self.logger.info("Starting get_player")
        if " " in name:
            raise InvalidName("The player name is Invalid")

        url = f"https://stats.pika-network.net/api/profile/{name}"

        async with aiohttp.ClientSession() as session:

            data = await fetch(session, url)
            player_data = json.loads(data)

            player_object = await build_player(player_data)

            return player_object

    async def get_player_leaderboard(
            self,
            name: str,
            server: str,
            interval: str,
            mode: typing.Optional[str] = None
    ) -> PlayerLeaderboard:
        """
        Returns a PlayerLeaderboard Object

        used to get data about a player's leaderboard stats.
        """

        if server not in ALLOWED_SERVERS:
            raise InvalidServerArgument(f"The server {server} does not exist!")

        self.logger.info("Starting get_player_leaderboard")
        if " " in name:
            raise InvalidName("The player name is Invalid")

        if mode is not None and server not in ["bedwars", "skywars"]:
            raise InvalidServerArgument(f"You cannot use the mode argument with the server {server}")

        if mode is None:
            url = f"https://stats.pika-network.net/api/profile/{name}/leaderboard?type={server}&interval={interval}"
        else:
            url = f"https://stats.pika-network.net/api/profile/{name}/leaderboard?type={server}&interval={interval}&mode={mode}"

        async with aiohttp.ClientSession() as session:

            data = await fetch(session, url)
            leaderboard_data = json.loads(data)

            leaderboard_object = await build_player_leaderboard(leaderboard_data)

            return leaderboard_object



"""
MIT License

Copyright (c) 2023 LetsChill

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from dataclasses import dataclass

from typing import (Optional, Union)


@dataclass
class ClanOwner:
    """
        A child object of Clan

        It contains information about the Owner of the clan.
        """
    username: str
    last_seen: int
    online: bool


@dataclass
class ClanMember:
    """
    A child object of Clan

    It contains information about a member in a clan.
    """
    username: str
    last_seen: int
    online: bool
    join_date: str


@dataclass
class Clan:
    """
    A child object of Player.

    It contains information about the Player's clan.
    """
    name: str
    tag: str
    current_trophies: str
    creation_time: str
    members: Union[list[ClanMember], list]
    owner: ClanOwner
    level: int
    experience: int
    total_experience: int


@dataclass
class Rank:
    """
    A child object of Player.

    It contains the information of the player's level, experience and the Rank Display of the minigames gamemode.
    """

    level: int
    experience: int
    percentage: float
    rank_display: str


@dataclass
class GameRank:
    """
    A child object of Player.

    It contains the information of the player's rank.
    """
    name: str
    display_name: str
    server: str
    season: Optional[str]


@dataclass
class Friend:
    """
    A child object of Player.

    It contains the information of a player's friend.
    """
    username: str
    last_seen: int
    online: bool


@dataclass
class Player:
    """
    A Player object.

    It contains the whole information of the player.
    """
    username: str
    friend_status: str
    discord_verified: bool
    last_seen: int
    ranks: list[GameRank]
    email_verified: bool
    discord_boosting: bool
    clan: Optional[Clan]
    rank: Rank
    friends: list[Friend]


@dataclass
class PlayerLeaderboardBase:
    total_players: int
    place: Union[int, None]
    value: Union[str, None]
    username: Union[str, None]


@dataclass
class PlayerLeaderboard:
    """
    A PlayerLeaderboard object

    It contains details about a player's leaderboard stats.
    """
    mobs_killed: Optional[PlayerLeaderboardBase] = None
    player_level: Optional[PlayerLeaderboardBase] = None
    deaths: Optional[PlayerLeaderboardBase] = None
    kills: Optional[PlayerLeaderboardBase] = None
    losses: Optional[PlayerLeaderboardBase] = None
    wins: Optional[PlayerLeaderboardBase] = None

    highest_killstreak: Optional[PlayerLeaderboardBase] = None
    current_killstreak: Optional[PlayerLeaderboardBase] = None
    highest_winstreak_reached: Optional[PlayerLeaderboardBase] = None

    balance: Optional[PlayerLeaderboardBase] = None

    blocks_placed: Optional[PlayerLeaderboardBase] = None
    blocks_broken: Optional[PlayerLeaderboardBase] = None

    gambling_skill_level: Optional[PlayerLeaderboardBase] = None
    swords_skill_level: Optional[PlayerLeaderboardBase] = None

    harvest_sugar_cane: Optional[PlayerLeaderboardBase] = None
    total_koths_won: Optional[PlayerLeaderboardBase] = None

    souls: Optional[PlayerLeaderboardBase] = None
    tokens: Optional[PlayerLeaderboardBase] = None
    tokens_obtained: Optional[PlayerLeaderboardBase] = None
    gems_obtained: Optional[PlayerLeaderboardBase] = None
    tokens_spent: Optional[PlayerLeaderboardBase] = None
    prestige: Optional[PlayerLeaderboardBase] = None
    exp_collected: Optional[PlayerLeaderboardBase] = None

    bow_kills: Optional[PlayerLeaderboardBase] = None
    arrow_kills: Optional[PlayerLeaderboardBase] = None
    melee_kills: Optional[PlayerLeaderboardBase] = None
    void_kills: Optional[PlayerLeaderboardBase] = None
    games_played: Optional[PlayerLeaderboardBase] = None
    arrows_shot: Optional[PlayerLeaderboardBase] = None
    arrows_hit: Optional[PlayerLeaderboardBase] = None
    beds_destroyed: Optional[PlayerLeaderboardBase] = None
    final_kills: Optional[PlayerLeaderboardBase] = None

    lumberjack_skill_level: Optional[PlayerLeaderboardBase] = None
    adventurer_skill_level: Optional[PlayerLeaderboardBase] = None
    farmer_skill_level: Optional[PlayerLeaderboardBase] = None
    archery_skill_level: Optional[PlayerLeaderboardBase] = None
    axes_skill_level: Optional[PlayerLeaderboardBase] = None
    special_ores_mined: Optional[PlayerLeaderboardBase] = None
    miner_skill_level: Optional[PlayerLeaderboardBase] = None
    wizard_skill_level: Optional[PlayerLeaderboardBase] = None
    banker_skill_level: Optional[PlayerLeaderboardBase] = None
    alchemist_skill_level: Optional[PlayerLeaderboardBase] = None
    lucky_skill_level: Optional[PlayerLeaderboardBase] = None
    drugs_skill_level: Optional[PlayerLeaderboardBase] = None
